@@ -80,6 +80,10 @@ custom_object::custom_object(wml::const_node_ptr node)
 	always_active_(wml::get_bool(node, "always_active", false)),
 	last_cycle_active_(0)
 {
+	if(node->has_attr("platform_area")) {
+		set_platform_area(rect(node->attr("platform_area")));
+	}
+
 	if(node->has_attr("x_schedule")) {
 		if(position_schedule_.get() == NULL) {
 			position_schedule_.reset(new position_schedule);
@@ -227,6 +231,13 @@ custom_object::custom_object(wml::const_node_ptr node)
 			add_particle_system(p, p);
 		}
 	}
+
+	FOREACH_WML_CHILD(light_node, node, "light") {
+		light_ptr new_light(light::create_light(*this, light_node));
+		if(new_light) {
+			lights_.push_back(new_light);
+		}
+	}
 }
 
 custom_object::custom_object(const std::string& type, int x, int y, bool face_right)
@@ -328,7 +339,6 @@ custom_object::custom_object(const custom_object& o) :
 custom_object::~custom_object()
 {
 	sound::stop_looped_sounds(this);
-	//std::cerr << "Object died:" << type_->id() << " " << this << " \n";
 }
 
 wml::node_ptr custom_object::write() const
@@ -339,6 +349,14 @@ wml::node_ptr custom_object::write() const
 		res->set_attr("position_scale_y", formatter() << position_scale_millis_->second);
 	}
 
+	if(platform_area_.get() != NULL) {
+		res->set_attr("platform_area", platform_area_->to_string());
+	}
+
+	if(always_active_) {
+		res->set_attr("always_active", "yes");
+	}
+
 	if(position_schedule_.get() != NULL) {
 		res->set_attr("schedule_speed", formatter() << position_schedule_->speed);
 		if(position_schedule_->x_pos.empty() == false) {
@@ -553,6 +571,10 @@ wml::node_ptr custom_object::write() const
 			res->set_attr("particles", systems);
 		}
 	}
+
+	foreach(const light_ptr& p, lights_) {
+		res->add_child(p->write());
+	}
 	
 	return res;
 }
@@ -875,7 +897,6 @@ void custom_object::process(level& lvl)
 
 	bool is_stuck = false;
 
-	//std::cerr << "velocity_y: " << velocity_y_ << "\n";
 	collide = false;
 	int move_left;
 	for(move_left = std::abs(effective_velocity_y); move_left > 0 && !collide && !type_->ignore_collide(); move_left -= 100) {
@@ -937,7 +958,6 @@ void custom_object::process(level& lvl)
 		}
 
 		if(collide) {
-			std::cerr << "collide y!\n";
 			break;
 		}
 	}
@@ -1102,7 +1122,6 @@ void custom_object::process(level& lvl)
 					//and one pixel below is walkable, then we move down by
 					//one pixel.
 					is_standing(lvl, &slope_standing_info);
-					std::cerr << "TRYING TO MOVE DOWN\n";
 					if(slope_standing_info.platform) {
 						set_y(y()+1);
 						if(!is_standing(lvl) || detect_collisions && entity_collides(lvl, *this, MOVE_DOWN)) {
@@ -1228,6 +1247,10 @@ void custom_object::process(level& lvl)
 			blur_.reset();
 		}
 	}
+
+	foreach(const light_ptr& p, lights_) {
+		p->process();
+	}
 }
 
 void custom_object::set_driver_position()
@@ -1388,6 +1411,10 @@ const solid_info* custom_object::calculate_solid() const
 
 const solid_info* custom_object::calculate_platform() const
 {
+	if(platform_solid_info_.get()) {
+		return platform_solid_info_.get();
+	}
+
 	return type_->platform().get();
 }
 
@@ -1592,6 +1619,15 @@ variant custom_object::get_value_by_slot(int slot) const
 		return call_stack(*this);
 	}
 
+	case CUSTOM_OBJECT_LIGHTS: {
+		std::vector<variant> result;
+		foreach(const light_ptr& p, lights_) {
+			result.push_back(variant(p.get()));
+		}
+
+		return variant(&result);
+	}
+
 	case CUSTOM_OBJECT_ALWAYS_ACTIVE: return variant(always_active_);
 	case CUSTOM_OBJECT_TAGS: return variant(tags_.get());
 	case CUSTOM_OBJECT_HAS_FEET: return variant(has_feet_);
@@ -1670,7 +1706,6 @@ void custom_object::set_value(const std::string& key, const variant& value)
 		time_in_frame_ = value.as_int()%frame_->duration();
 	} else if(key == "time_in_animation_delta") {
 		time_in_frame_delta_ = value.as_int();
-		std::cerr << "SET TIME IN ANIMATION_DELTA: " << time_in_frame_delta_ << "\n";
 	} else if(key == "x") {
 		const int start_x = centi_x();
 		set_x(value.as_int());
@@ -1899,7 +1934,6 @@ void custom_object::set_value_by_slot(int slot, const variant& value)
 		break;
 	case CUSTOM_OBJECT_TIME_IN_ANIMATION_DELTA:
 		time_in_frame_delta_ = value.as_int();
-		std::cerr << "SET TIME IN ANIMATION_DELTA: " << time_in_frame_delta_ << "\n";
 		break;
 	case CUSTOM_OBJECT_ANIMATION:
 		set_frame(value.as_string());
@@ -2151,6 +2185,17 @@ void custom_object::set_value_by_slot(int slot, const variant& value)
 		break;
 	}
 
+	case CUSTOM_OBJECT_LIGHTS: {
+		lights_.clear();
+		for(int n = 0; n != value.num_elements(); ++n) {
+			light* p = value[n].try_convert<light>();
+			if(p) {
+				lights_.push_back(light_ptr(p));
+			}
+		}
+		break;
+	}
+
 	case CUSTOM_OBJECT_SOLID_DIMENSIONS_IN:
 	case CUSTOM_OBJECT_SOLID_DIMENSIONS_NOT_IN: {
 		unsigned int solid = 0, weak = 0;
@@ -2235,6 +2280,15 @@ void custom_object::set_value_by_slot(int slot, const variant& value)
 		break;
 	}
 
+	case CUSTOM_OBJECT_PLATFORM_AREA: {
+		ASSERT_GE(value.num_elements(), 3);
+		ASSERT_LE(value.num_elements(), 4);
+
+		set_platform_area(rect(value[0].as_int(), value[1].as_int(), value[2].as_int(), 1));
+		calculate_solid_rect();
+		break;
+	}
+
 	default:
 		break;
 
@@ -2412,7 +2466,6 @@ void custom_object::move_to_standing(level& lvl)
 	}
 	
 	set_pos(x(), start_y);
-	std::cerr << "MOVE_TO_STANDING FAILED\n";
 }
 
 
@@ -2751,6 +2804,28 @@ bool custom_object::allow_level_collisions() const
 	return type_->use_image_for_collisions();
 }
 
+void custom_object::set_platform_area(const rect& area)
+{
+	if(area.w() <= 0 || area.h() <= 0) {
+		platform_area_.reset();
+		platform_solid_info_ = const_solid_info_ptr();
+	} else {
+		platform_area_.reset(new rect(area));
+		platform_solid_info_ = solid_info::create_platform(area);
+	}
+}
+
+void custom_object::shift_position(int x, int y)
+{
+	entity::shift_position(x, y);
+	if(activation_area_.get() != NULL) {
+		activation_area_.reset(new rect(activation_area_->x() + x,
+		                                activation_area_->y() + y,
+										activation_area_->w(),
+										activation_area_->h()));
+	}
+}
+
 BENCHMARK(custom_object_spike) {
 	static level* lvl = NULL;
 	if(!lvl) {	
