@@ -1,6 +1,7 @@
 #include <boost/bind.hpp>
 #include <iostream>
 #include <map>
+#include <vector>
 #include <string>
 #include <time.h>
 
@@ -42,6 +43,7 @@
 #include "wml_utils.hpp"
 #include "wml_writer.hpp"
 #include "preferences.hpp"
+#include "settings_dialog.hpp"
 
 using namespace game_logic;
 
@@ -194,18 +196,29 @@ class sound_command : public entity_command_callable
 {
 public:
 	explicit sound_command(const std::string& name, const bool loops)
-	  : name_(name), loops_(loops)
+	  : names_(util::split(name)), loops_(loops)
 	{}
 	virtual void execute(level& lvl, entity& ob) const {
 		if(loops_){
-			sound::play_looped(name_, &ob);
+			if (names_.empty() == false){
+				int randomNum = rand()%names_.size();  //like a 1d-size die
+				if(names_[randomNum].empty() == false) {
+					sound::play_looped(names_[randomNum], &ob);
+				}
+			}
+			
 		}else{
-			std::cerr << "PLAY SOUND COMMAND: " << name_ << "\n";
-			sound::play(name_, &ob);
+			//std::cerr << "PLAY SOUND COMMAND: " << name_ << "\n";
+			if (names_.empty() == false){
+				int randomNum = rand()%names_.size();  //like a 1d-size die
+				if(names_[randomNum].empty() == false) {
+					sound::play(names_[randomNum], &ob);
+				}
+			}
 		}
 	}
 private:
-	std::string name_;
+	std::vector<std::string> names_;
 	bool loops_;
 };
 
@@ -920,21 +933,75 @@ FUNCTION_DEF(transient_speech_dialog, 1, -1, "transient_speech_dialog(...): sche
 		}
 	}
 
+	//we add the dialogs in reverse order to the level to make it get
+	//said in the correct order.
+	std::reverse(result.begin(), result.end());
+
 	return variant(&result);
 END_FUNCTION_DEF(transient_speech_dialog)
 
 namespace {
+//if this variable is set we are in a controlled dialog sequence skipping
+//scope and the end of dialog shouldn't automatically end skipping dialog.
+bool skipping_dialog_sequence = false;
+
 struct in_dialog_setter {
+	bool was_in_dialog_;
 	level& lvl_;
 	in_dialog_setter(level& lvl) : lvl_(lvl) { 
+		was_in_dialog_ = lvl_.in_dialog();
 		lvl_.set_in_dialog(true);
 	}
 	~in_dialog_setter() {
-		lvl_.set_in_dialog(false);
+		lvl_.set_in_dialog(was_in_dialog_);
+		if(!was_in_dialog_ && !skipping_dialog_sequence) {
+			end_skipping_game();
+		}
 	}
 };
+
+struct speech_dialog_scope {
+	level& lvl_;
+	boost::shared_ptr<speech_dialog> dialog_;
+
+	speech_dialog_scope(level& lvl, boost::shared_ptr<speech_dialog> dialog)
+	  : lvl_(lvl), dialog_(dialog)
+	{
+		lvl_.add_speech_dialog(dialog_);
+	}
+
+	~speech_dialog_scope()
+	{
+		lvl_.remove_speech_dialog();
+	}
+};
+
 }
 
+class skip_sequence_command : public custom_object_command_callable {
+	bool skip_on_;
+public:
+	explicit skip_sequence_command(bool skip_on)
+	  : skip_on_(skip_on)
+	{}
+
+	virtual void execute(level& lvl, custom_object& ob) const {
+		skipping_dialog_sequence = skip_on_;
+		if(!skip_on_) {
+			end_skipping_game();
+		}
+	}
+};
+
+FUNCTION_DEF(begin_skip_dialog_sequence, 0, 0, "begin_skip_dialog_sequence(): command that will cause everything up until the next time end_skip_dialog_sequence() is called to be considered a single storyline sequence. If the player selects to skip the sequence between now and then everything up until the call to end_skip_dialog_sequence() will be skipped.")
+	return variant(new skip_sequence_command(true));
+END_FUNCTION_DEF(begin_skip_dialog_sequence)
+
+FUNCTION_DEF(end_skip_dialog_sequence, 0, 0, "end_skip_dialog_sequence(): ends the sequence begun with begin_skip_dialog_sequence().")
+	std::cerr << "ENDDIALOG\n";
+	return variant(new skip_sequence_command(false));
+END_FUNCTION_DEF(end_skip_dialog_sequence)
+
 class speech_dialog_command : public custom_object_command_callable {
 public:
 	explicit speech_dialog_command(const std::vector<variant>& args, bool paused=false)
@@ -943,12 +1010,10 @@ public:
 	virtual void execute(level& lvl, custom_object& ob) const {
 //		pause_scope pauser;
 
-		if(g_in_speech_dialog) {
-			return;
-		}
-
-		foreach(const entity_ptr& e, lvl.get_chars()) {
-			e->handle_event(OBJECT_EVENT_BEGIN_DIALOG);
+		if(!g_in_speech_dialog) {
+			foreach(const entity_ptr& e, lvl.get_chars()) {
+				e->handle_event(OBJECT_EVENT_BEGIN_DIALOG);
+			}
 		}
 
 		formula_profiler::suspend_scope profiler_suspend;
@@ -961,15 +1026,21 @@ public:
 	}
 
 private:
-	void execute_commands(level& lvl, custom_object& ob, const std::vector<variant>& commands) const {
-		in_speech_dialog_tracker dialog_tracker;
+	static settings_dialog menu_button_;
 	
+	void execute_commands(level& lvl, custom_object& ob, const std::vector<variant>& commands) const {
+		menu_button_.reset();
+		in_speech_dialog_tracker dialog_tracker;
+
+		boost::shared_ptr<speech_dialog> d(new speech_dialog());
+		speech_dialog_scope dialog_scope(lvl, d);
+
 		foreach(variant var, commands) {
 			if(var.is_callable()) {
 				const_entity_ptr e = var.try_convert<entity>();
 				if(e) {
 					std::cerr << "set speaker...\n";
-					dialog_.set_speaker_and_flip_side(e);
+					d->set_speaker_and_flip_side(e);
 				}
 
 				const entity_command_callable* entity_cmd = var.try_convert<const entity_command_callable>();
@@ -994,11 +1065,21 @@ private:
 					continue;
 				}
 
+				bool is_default = false;
+				int default_option = -1;
+
 				std::vector<variant> option_commands;
 				std::vector<std::string> options;
 				std::vector<std::string> message;
 				for(int n = 0; n != var.num_elements(); ++n) {
-					if(message.empty() == false && var[n].is_list()) {
+					if(var[n].is_string() && var[n].as_string() == "default_skip") {
+						is_default = true;
+					} else if(message.empty() == false && var[n].is_list()) {
+						if(is_default) {
+							default_option = n;
+							is_default = false;
+						}
+
 						options.push_back(message.back());
 						message.pop_back();
 						option_commands.push_back(var[n]);
@@ -1007,8 +1088,8 @@ private:
 					}
 				}
 
-				dialog_.set_text(message);
-				dialog_.set_options(options);
+				d->set_text(message);
+				d->set_options(options);
 
 				bool done = false;
 				while(!done) {
@@ -1019,14 +1100,22 @@ private:
 
 					SDL_Event event;
 					while(SDL_PollEvent(&event)) {
+#if TARGET_IPHONE_SIMULATOR || TARGET_OS_IPHONE
+						// the user event gets handled the same as pressing escape
+						if (menu_button_.handle_event(event))
+						{
+							event.type = SDL_USEREVENT;
+						}
+#endif
 						switch(event.type) {
 						case SDL_QUIT:
 							throw interrupt_game_exception();
+						case SDL_USEREVENT:
 						case SDL_KEYDOWN:
-							if(event.key.keysym.sym == SDLK_ESCAPE) {
-								PAUSE_GAME_RESULT result = show_pause_game_dialog();
-								if(result != PAUSE_GAME_CONTINUE) {
-									throw interrupt_game_exception(result);
+							if(event.key.keysym.sym == SDLK_ESCAPE || event.type == SDL_USEREVENT) {
+								begin_skipping_game();
+								if(default_option != -1) {
+									d->set_option_selected(default_option);
 								}
 								break;
 							}
@@ -1034,32 +1123,36 @@ private:
 						case SDL_MOUSEBUTTONDOWN:
 						case SDL_MOUSEBUTTONUP:
 						case SDL_MOUSEMOTION:
-							done = done || dialog_.key_press(event);
+							done = done || d->key_press(event);
 							break;
 						}
 					}
 
-					done = done || dialog_.process();
+					if(paused_)
+						done = done || d->process();
+					else if(!lvl.current_speech_dialog())
+						done = true;
 					draw(lvl);
 				}
 
 				if(options.empty() == false) {
-					const int index = dialog_.option_selected();
+					const int index = d->option_selected();
 					if(index >= 0 && index < option_commands.size()) {
 						dialog_tracker.cancel();
 						ob.execute_command(option_commands[index]);
 					}
 				}
 
-				dialog_.set_options(std::vector<std::string>());
+				d->set_options(std::vector<std::string>());
 			}
 		}
 	}
 
 	void draw(const level& lvl) const {
 		draw_scene(lvl, last_draw_position(), &lvl.player()->get_entity());
-
-		dialog_.draw();
+#if TARGET_IPHONE_SIMULATOR || TARGET_OS_IPHONE
+		menu_button_.draw(in_speech_dialog());
+#endif
 
 		SDL_GL_SwapBuffers();
 		SDL_Delay(20);
@@ -1067,10 +1160,11 @@ private:
 
 	std::vector<variant> args_;
 
-	mutable speech_dialog dialog_;
 	bool paused_;
 };
 
+settings_dialog speech_dialog_command::menu_button_ = settings_dialog();
+
 FUNCTION_DEF(speech_dialog, 1, -1, "speech_dialog(...): schedules a sequence of speech dialogs to be shown modally. Arguments may include a list of strings, which contain text. An integer which sets the duration of the dialog. An object which sets the speaker. A string by itself indicates an option that should be shown for the player to select from. A string should be followed by a list of commands that will be executed should the player choose that option.")
 	std::vector<variant> v;
 	for(int n = 0; n != args().size(); ++n) {
@@ -1540,6 +1634,12 @@ FUNCTION_DEF(end_script, 0, 0, "end_script(): ends the most recent script to hav
 	return variant(new end_script_command);
 END_FUNCTION_DEF(end_script)
 
+FUNCTION_DEF(circle_light, 2, 2, "circle_light(object, radius): creates a circle of light with the given radius")
+	return variant(new circle_light(
+	          *args()[0]->evaluate(variables).convert_to<custom_object>(),
+			  args()[1]->evaluate(variables).as_int()));
+END_FUNCTION_DEF(circle_light)
+
 class add_particles_command : public custom_object_command_callable {
 public:
 	add_particles_command(const std::string& id, const std::string& type)
@@ -1623,6 +1723,54 @@ FUNCTION_DEF(swallow_event, 0, 0, "swallow_event(): when used in an instance-spe
 	return variant(new swallow_object_command_callable);
 END_FUNCTION_DEF(swallow_event)
 
+class add_level_module_command : public entity_command_callable {
+	std::string lvl_;
+	int x_, y_;
+public:
+	add_level_module_command(const std::string& lvl, int x, int y)
+	  : lvl_(lvl), x_(x), y_(y)
+	{}
+
+	virtual void execute(level& lvl, entity& ob) const {
+		lvl.add_sub_level(lvl_, x_, y_);
+	}
+};
+
+FUNCTION_DEF(add_level_module, 3, 3, "add_level_module(string lvl, int xoffset, int yoffset): adds the level module with the given level id at the given offset")
+	return variant(new add_level_module_command(args()[0]->evaluate(variables).string_cast(), args()[1]->evaluate(variables).as_int(), args()[2]->evaluate(variables).as_int()));
+END_FUNCTION_DEF(add_level_module)
+
+class remove_level_module_command : public entity_command_callable {
+	std::string lvl_;
+public:
+	explicit remove_level_module_command(const std::string& lvl) : lvl_(lvl)
+	{}
+
+	virtual void execute(level& lvl, entity& ob) const {
+		lvl.remove_sub_level(lvl_);
+	}
+};
+
+FUNCTION_DEF(remove_level_module, 1, 1, "remove_level_module(string lvl): removes the given level module")
+	return variant(new remove_level_module_command(args()[0]->evaluate(variables).string_cast()));
+END_FUNCTION_DEF(remove_level_module)
+
+class shift_level_position_command : public entity_command_callable {
+	int xoffset_, yoffset_;
+public:
+
+	shift_level_position_command(int xoffset, int yoffset)
+	  : xoffset_(xoffset), yoffset_(yoffset) {}
+
+	virtual void execute(level& lvl, entity& ob) const {
+		lvl.adjust_level_offset(xoffset_, yoffset_);
+	}
+};
+
+FUNCTION_DEF(cosmic_shift, 2, 2, "cosmic_shift(int xoffset, int yoffet): adjust position of all objects and tiles in the level by the given offset")
+	return variant(new shift_level_position_command(args()[0]->evaluate(variables).as_int(), args()[1]->evaluate(variables).as_int()));
+END_FUNCTION_DEF(cosmic_shift)
+
 class custom_object_function_symbol_table : public function_symbol_table
 {
 public:
@@ -1653,6 +1801,8 @@ expression_ptr custom_object_function_symbol_table::create_function(
 
 } //namespace
 
+bool in_speech_dialog () {return g_in_speech_dialog > 0;}
+
 function_symbol_table& get_custom_object_functions_symbol_table()
 {
 	static custom_object_function_symbol_table table;
@@ -1684,9 +1834,9 @@ UTILITY(document_object_functions) {
 	foreach(std::string s, function_helpstrings()) {
 		std::string::iterator i = std::find(s.begin(), s.end(), ':');
 		if(i != s.end()) {
-			s = "<b>" + std::string(s.begin(), i) + "</b>" + std::string(i, s.end());
+			s = "{{{ " + std::string(s.begin(), i) + " }}}" + std::string(i, s.end());
 		}
-		s = "<p>" + s + "</p>";
+		s = "  * " + s;
 		std::cout << s << "\n";
 	}
 }
