@@ -20,6 +20,7 @@
 #include "inventory.hpp"
 #include "joystick.hpp"
 #include "level_runner.hpp"
+#include "light.hpp"
 #include "load_level.hpp"
 #include "message_dialog.hpp"
 #include "object_events.hpp"
@@ -39,6 +40,7 @@
 
 namespace {
 const int FrameTimeInMillis = 20;
+int skipping_game = 0;
 
 int global_pause_time;
 
@@ -59,13 +61,21 @@ void transition_scene(level& lvl, screen_position& screen_pos, bool transition_o
 		lvl.player()->get_entity().set_invisible(true);
 	}
 
+	const int start_time = SDL_GetTicks();
+
 	for(int n = 0; n <= 20; ++n) {
 //		lvl.process();
 
 		draw_fn(lvl, screen_pos, transition_out ? (n/20.0) : (1 - n/20.0));
 
 		SDL_GL_SwapBuffers();
-		SDL_Delay(FrameTimeInMillis);
+
+		const int target_end_time = start_time + (n+1)*FrameTimeInMillis;
+		const int current_time = SDL_GetTicks();
+		const int skip_time = target_end_time - current_time;
+		if(skip_time > 0) {
+			SDL_Delay(skip_time);
+		}
 	}
 	
 	if(lvl.player()) {
@@ -91,96 +101,33 @@ bool calculate_stencil_buffer_available() {
 	return stencil_buffer_bits > 0;	
 }
 
-bool is_stencil_buffer_available() {
-	static const bool result = calculate_stencil_buffer_available();
-	return result;
-}
-
 void iris_scene(const level& lvl, screen_position& screen_pos, float amount) {
-	if(!is_stencil_buffer_available()) {
-		//there's no stencil buffer available, so resort to a fade
-		//effect instead.
-		fade_scene(lvl, screen_pos, amount);
-		return;
-	}
-
 	if(lvl.player() == NULL) {
 		return;
 	}
+	
+	const_entity_ptr player = &lvl.player()->get_entity();
+	std::map<const_entity_ptr, std::vector<light_ptr> > lights;
+	lights[player].push_back(light_ptr(new circle_light(*dynamic_cast<const custom_object*>(player.get()), (1.0-amount)*500)));
 
-	//Enable the stencil buffer.
-	glEnable(GL_STENCIL_TEST);
-	glClear(GL_STENCIL_BUFFER_BIT);
-
-	//Set things up so we will draw to the stencil buffer, not to the screen.
-	glStencilFunc(GL_NEVER, 1, 1);
-	glStencilOp(GL_REPLACE, GL_KEEP, GL_KEEP);
-
-	glPushMatrix();
-
-	glTranslatef(-screen_pos.x/100, -screen_pos.y/100, 0);
-
-	point pos = lvl.player()->get_entity().midpoint();
-
-	//we want to make the circle shrink quickly at first, but then slow
-	//as it gets smaller, so we use the square of the amount to achieve this.
-	amount = (1.0 - amount)*(1.0 - amount);
-
-	//Draw a circle.
-	const float radius = amount*500;
-	std::vector<GLfloat>& varray = graphics::global_vertex_array();
-	varray.clear();
-	varray.push_back(pos.x);
-	varray.push_back(pos.y);
-	for(double angle = 0; angle < 3.1459*2.0; angle += 0.01) {
-		const double x = pos.x + radius*cos(angle);
-		const double y = pos.y + radius*sin(angle);
-		varray.push_back(x);
-		varray.push_back(y);
+	foreach(entity_ptr e, lvl.get_active_chars()) {
+		e->swap_lights(lights[e]);
 	}
 
-	glVertexPointer(2, GL_FLOAT, 0, &varray.front());
-	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-	glDrawArrays(GL_TRIANGLE_FAN, 0, varray.size()/2);
-	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
+	const bool dark_value = const_cast<level&>(lvl).set_dark(true);
 
-	glPopMatrix();
+	if(amount >= 0.99) {
+		SDL_Rect rect = {0, 0, graphics::screen_width(), graphics::screen_height()};
+		graphics::draw_rect(rect, graphics::color_black());
+	} else {
+		draw_scene(lvl, screen_pos);
+	}
 
-	//Now we've set the stencil to a circle, set things up so that the stencil
-	//will be used to draw only within the circle.
-	glStencilFunc(GL_EQUAL, 1, 1);
-	glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
-	
-	draw_scene(lvl, screen_pos);
+	const_cast<level&>(lvl).set_dark(dark_value);
 
-	//Now, reverse the rule; use the stencil to draw everything outside the circle - in this case, jet black.
-	glStencilFunc(GL_NOTEQUAL, 1, 1);
-	
-	GLfloat varray2[8];
-
-	//populate the four screen corners
-	varray2[0] = 0;
-	varray2[1] = 0;
-	
-	varray2[2] = graphics::screen_width();
-	varray2[3] = 0;
-	
-	varray2[4] = 0;
-	varray2[5] = graphics::screen_height();
-	
-	varray2[6] = graphics::screen_width();
-	varray2[7] = graphics::screen_height();
-	
-	glVertexPointer(2, GL_FLOAT, 0, &varray2);
-	glColor4ub(0, 0, 0, 255);
-	glDisable(GL_TEXTURE_2D);
-
-	glDrawArrays(GL_TRIANGLE_STRIP, 0, sizeof(varray2)/sizeof(GLfloat)/2);
-
-	glEnable(GL_TEXTURE_2D);
-
-	
-	glDisable(GL_STENCIL_TEST);
+	foreach(entity_ptr e, lvl.get_active_chars()) {
+		e->swap_lights(lights[e]);
+	}
 }
 
 void show_end_game()
@@ -221,6 +168,18 @@ void show_end_game()
 
 }
 
+void begin_skipping_game() {
+	++skipping_game;
+}
+
+void end_skipping_game() {
+	skipping_game = 0;
+}
+
+bool is_skipping_game() {
+	return skipping_game > 0;
+}
+
 level_runner::level_runner(boost::intrusive_ptr<level>& lvl, std::string& level_cfg, std::string& original_level_cfg)
   : lvl_(lvl), level_cfg_(level_cfg), original_level_cfg_(original_level_cfg)
 {
@@ -301,9 +260,9 @@ bool level_runner::play_cycle()
 		}
 	}
 
-	//record player movement every 5 cycles.
+	//record player movement every 500 cycles on average.
 #if !TARGET_OS_IPHONE
-	if(cycle%5 == 0 && lvl_->player()) {
+	if(rand()%500 == 0 && lvl_->player()) {
 		point p = lvl_->player()->get_entity().midpoint();
 
 		if(last_stats_point_level_ == lvl_->id()) {
@@ -402,7 +361,7 @@ bool level_runner::play_cycle()
 				transition_scene(*lvl_, last_draw_position(), true, flip_scene);
 			} else if(transition == "instant") {
 				//do nothing
-			} else if(transition != "fade" && is_stencil_buffer_available()) {
+			} else if(transition != "fade") {
 				transition_scene(*lvl_, last_draw_position(), true, iris_scene);
 			} else {
 				preload_level(level_cfg_);
@@ -480,25 +439,44 @@ bool level_runner::play_cycle()
 #endif
 			switch(event.type) {
 			case SDL_QUIT: {
-#if TARGET_IPHONE_SIMULATOR || TARGET_OS_IPHONE
-				wml::node_ptr node = lvl_->write();
-				if(sound::current_music().empty() == false) {
-					node->set_attr("music", sound::current_music());
-				}
-
-				sys::write_file(preferences::auto_save_file_path(), wml::output(node));
-				sys::write_file(std::string(preferences::auto_save_file_path()) + ".stat", "1");
-#endif
+				stats::record_event(lvl_->id(), stats::record_ptr(new stats::quit_record(lvl_->player()->get_entity().midpoint())));
 				done = true;
 				quit_ = true;
 				break;
 			}
 			case SDL_VIDEORESIZE: {
 				const SDL_ResizeEvent* const resize = reinterpret_cast<SDL_ResizeEvent*>(&event);
+				int width = resize->w;
+				int height = resize->h;
 
-				preferences::set_actual_screen_width(resize->w);
-				preferences::set_actual_screen_height(resize->h);
-				SDL_SetVideoMode(resize->w,resize->h,0,SDL_OPENGL|SDL_RESIZABLE|(preferences::fullscreen() ? SDL_FULLSCREEN : 0));
+				const int aspect = (preferences::actual_screen_width()*1000)/preferences::actual_screen_height();
+
+				if(preferences::actual_screen_width()*preferences::actual_screen_height() < width*height) {
+					//making the window larger
+					if((height*aspect)/1000 > width) {
+						width = (height*aspect)/1000;
+					} else if((height*aspect)/1000 < width) {
+						height = (width*1000)/aspect;
+					}
+				} else {
+					//making the window smaller
+					if((height*aspect)/1000 > width) {
+						height = (width*1000)/aspect;
+					} else if((height*aspect)/1000 < width) {
+						width = (height*aspect)/1000;
+					}
+				}
+
+				//make sure we don't have some ugly fractional aspect ratio
+				while((width*1000)/height != aspect) {
+					++width;
+					height = (width*1000)/aspect;
+				}
+
+				preferences::set_actual_screen_width(width);
+				preferences::set_actual_screen_height(height);
+
+				graphics::set_video_mode(width, height);
 				continue;
 			}
 #if TARGET_IPHONE_SIMULATOR || TARGET_OS_IPHONE
@@ -572,7 +550,7 @@ bool level_runner::play_cycle()
 					graphics::texture::clear_textures();
 				} else if(key == SDLK_f && mod & KMOD_CTRL) {
 					preferences::set_fullscreen(!preferences::fullscreen());
-					SDL_SetVideoMode(graphics::screen_width(),graphics::screen_height(),0,SDL_OPENGL|(preferences::fullscreen() ? SDL_FULLSCREEN : 0));
+					graphics::set_video_mode(graphics::screen_width(), graphics::screen_height());
 				}
 				break;
 			}
@@ -621,20 +599,25 @@ bool level_runner::play_cycle()
 	const int start_draw = SDL_GetTicks();
 	if(start_draw < desired_end_time || nskip_draw_ >= MaxSkips) {
 		lvl_->process_draw();
-		draw_scene(*lvl_, last_draw_position());
+
+		draw_scene(*lvl_, last_draw_position(), NULL, !is_skipping_game());
+
 		performance_data perf = { current_fps_, current_cycles_, current_delay_, current_draw_, current_process_, current_flip_, cycle, current_events_, profiling_summary_ };
 #if TARGET_IPHONE_SIMULATOR || TARGET_OS_IPHONE
-		settings_dialog.draw();
+		settings_dialog.draw(in_speech_dialog());
 #endif
 		
-		if(preferences::show_fps()) {
+		if(!is_skipping_game() && preferences::show_fps()) {
 			draw_fps(*lvl_, perf);
 		}
 
 		next_draw_ += (SDL_GetTicks() - start_draw);
 
 		const int start_flip = SDL_GetTicks();
-		SDL_GL_SwapBuffers();
+		if(!is_skipping_game()) {
+			SDL_GL_SwapBuffers();
+		}
+
 		next_flip_ += (SDL_GetTicks() - start_flip);
 		++next_fps_;
 		nskip_draw_ = 0;
@@ -668,10 +651,33 @@ bool level_runner::play_cycle()
 	const int raw_wait_time = desired_end_time - SDL_GetTicks();
 	const int wait_time = std::max<int>(1, desired_end_time - SDL_GetTicks());
 	next_delay_ += wait_time;
-	if (wait_time != 1) SDL_Delay(wait_time);
+	if (wait_time != 1 && !is_skipping_game()) {
+		SDL_Delay(wait_time);
+	}
+	
+	if(is_skipping_game()) {
+		const int adjust_time = desired_end_time - SDL_GetTicks();
+		if(adjust_time > 0) {
+			pause_time_ -= adjust_time;
+		}
+	}
 
 	if (!paused) ++cycle;
 
+	
+#if TARGET_IPHONE_SIMULATOR || TARGET_OS_IPHONE
+	if (quit_)
+	{
+		wml::node_ptr node = lvl_->write();
+		if(sound::current_music().empty() == false) {
+			node->set_attr("music", sound::current_music());
+		}
+		
+		sys::write_file(preferences::auto_save_file_path(), wml::output(node));
+		sys::write_file(std::string(preferences::auto_save_file_path()) + ".stat", "1");
+	}
+#endif
+	
 	return !quit_;
 }
 
