@@ -36,6 +36,8 @@ public:
 	bool can_interact() const { return can_interact_ != 0; }
 
 private:
+	bool on_platform() const;
+
 	int walk_up_or_down_stairs() const;
 
 	virtual void process(level& lvl);
@@ -53,6 +55,8 @@ private:
 	bool underwater_controls_;
 
 	int can_interact_;
+	
+	bool reverse_ab_;
 
 	boost::scoped_ptr<controls::local_controls_lock> control_lock_;
 
